@@ -108,7 +108,8 @@ static struct variable hr_variables[] = {
 
 static oid hr_base [] = { MIB, 25 };
 
-
+/* max number of cpu's we support: */
+#define HR_MAX_CPU	32
 
 /* initial load device index (points into hrDeviceEntry) */
 static unsigned long hr_initial_load_dev = 0;
@@ -126,28 +127,59 @@ static unsigned long hr_initial_load_dev = 0;
 #define NROFSAMPLES 6		/* number of samples per minute;
 				   minimum 2 */
 
-static int interval = 60/NROFSAMPLES;
-static long int idleJiffies[NROFSAMPLES+1];
-				/* cyclic array with idle jiffies sampled */
-static time_t sampleTimes[NROFSAMPLES+1]; /* time of each sample */
+static int interval = 60 / NROFSAMPLES;
+/* cyclic array with idle jiffies sampled */
+static unsigned long idleJiffies [NROFSAMPLES + 1] [HR_MAX_CPU + 1];
+/* time of each sample */
+static time_t sampleTimes [NROFSAMPLES + 1];
 static int sindex;		/* index of the most recent sample */
 
 static void update_HrProcessorLoad();
 
-
-static long int 
-get_IdleJiffies()
+/*
+ * collect idle times for all cpu's together and for smp systems for 
+ * each one:
+ */
+static void
+get_IdleJiffies (data)
+     time_t *data;
 {
     FILE *in;
-    static long int idle = 0;
+    int i;
 
-    if((in = fopen("/proc/stat", "r"))) {
-	(void) fscanf(in, "cpu %*d %*d %*d %ld", &idle);
-	fclose(in);
+    /* clear old values: */
+    for (i = 0; i < HR_MAX_CPU + 1; i++) {
+      data [i] = 0;
+    }
+
+    if ((in = fopen("/proc/stat", "r"))) {
+      char tmp [256];
+      unsigned long idle = 0;
+      int idx;
+
+      while (fgets (tmp, sizeof(tmp), in)) {
+	int rc = 0;
+	if (! strncmp (tmp, "cpu ", 4) 
+	    && (1 == sscanf (tmp, "cpu %*u %*u %*u %lu", &idle))) {
+	  data [0] = idle;
+	} else if (! strncmp (tmp, "cpu", 3)
+		   && (tmp[3] >= '0' && tmp[3] <= '9')
+		   && 2 == (rc = sscanf (tmp + 3, "%d %*u %*u %*u %lu",
+					 &idx, &idle))) {
+	  if (idx >= 0 && idx <= HR_MAX_CPU) {
+	    data [idx + 1] = idle;
+	  }
+	}
+      }
+      fclose(in);
     } else {
-	fprintf(stderr, "snmpd: cannot open /proc/stat - please make sure /proc is mounted.\n");
+      static int reported = 0;
+      if (! reported) {
+	fprintf(stderr, 
+    "snmpd: cannot open /proc/stat - please make sure /proc is mounted.\n");
+	reported = 1;
+      }
     }
-    return idle;
 }
 
 
@@ -156,11 +188,10 @@ hr_init()
 {
     int i;
     time_t t = time(0);
-    long int ij = get_IdleJiffies();
     
     for (i=0; i <= NROFSAMPLES ;i++) {
-	idleJiffies[i] = ij;
-	sampleTimes[i] = t;
+      get_IdleJiffies (idleJiffies [i]);
+      sampleTimes[i] = t;
     }
     sindex = 0;
 
@@ -184,17 +215,22 @@ update_HrProcessorLoad()
     gettimeofday (&tv, 0);
     fprintf (stderr, "tv: %d.%06d...\n", tv.tv_sec, tv.tv_usec);
 #endif
-    sindex = (sindex+1) % (NROFSAMPLES+1);
-    sampleTimes[sindex] = time((time_t *) 0);
-    idleJiffies[sindex] = get_IdleJiffies();
+    sindex = (sindex + 1) % (NROFSAMPLES + 1);
+    sampleTimes[sindex] = time ((time_t *) 0);
+    get_IdleJiffies (idleJiffies[sindex]);
 }
 
+/*
+ * return current load; idx == 0 is overall info (single processor)
+ * else idx > 0 is a specific cpu if multoprocessor.
+ */
 static int 
-get_HrProcessorLoad()
+get_HrProcessorLoad (idx)
+     int idx;
 {
-    long int elapsedJiffies, nonIdleJiffies;
+    unsigned long elapsedJiffies, nonIdleJiffies;
     sigset_t currentBlocked, previousBlocked;
-    int oldestIndex = (sindex+1) % (NROFSAMPLES+1);
+    int oldestIndex = (sindex + 1) % (NROFSAMPLES + 1);
     
     /* prevent signal handler from clobbering the data structures */
     (void) sigemptyset(&currentBlocked);
@@ -202,21 +238,22 @@ get_HrProcessorLoad()
     sigprocmask(SIG_BLOCK, &currentBlocked, &previousBlocked);
     
     /* actual computation */
-    elapsedJiffies = (sampleTimes[sindex] - sampleTimes[oldestIndex])*100;
+    elapsedJiffies = (sampleTimes[sindex] - sampleTimes[oldestIndex]) * 100;
     nonIdleJiffies =
-	elapsedJiffies - (idleJiffies[sindex] - idleJiffies[oldestIndex]);
+	elapsedJiffies - (idleJiffies[sindex][idx] 
+			  - idleJiffies[oldestIndex][idx]);
 
     /* restore signal mask */
     sigprocmask(SIG_SETMASK, &previousBlocked, NULL);
     
-    /*
-      printf("non idle: %d elapsed: %d\n", nonIdleJiffies, elapsedJiffies);
-      */
+    /** fprintf (stderr, "idx %d: non idle: %lu elapsed: %lu\n", 
+	     idx, nonIdleJiffies, elapsedJiffies); **/
     
-    if (elapsedJiffies > 0)
+    if (elapsedJiffies > 0) {
 	return (int) ((100 * nonIdleJiffies) / elapsedJiffies + 0.5);
-    else
+    } else {
 	return 0;
+    }
 }
 
 
@@ -520,7 +557,6 @@ var_hr(vp, name, length, exact, var_len, write_method)
      */
 
     /* now we only support processor devices for now: */
-#define HR_MAX_CPU	16
 #define HR_MAX_DEV	(HR_MAX_CPU + 0)
 
     /* per default one running processor should be present: */
@@ -716,10 +752,40 @@ var_hr(vp, name, length, exact, var_len, write_method)
 	    if (i >= HR_N_STOR) {
 	        if (! in || ! fgets (line, sizeof (line), in))
 		  break; 
+
+	/*** XXX: what to use of them:
+/dev/ram0		First RAM disk
+/dev/fd0		Controller 1, drive 1 autodetect
+/dev/hda		Master: whole disk (or CD-ROM)
+/dev/loop0	First loopback device
+/dev/sda		First SCSI disk whole disk
+/dev/md0		First metadisk group
+/dev/sr0		First SCSI CD-ROM
+/dev/xda		First XT disk whole disk
+/dev/dos_hda	First BIOS harddrive whole disk
+/dev/sonycd	Sony CDU-31a CD-ROM
+/dev/gscd		GoldStar CD-ROM
+/dev/optcd	Optics Storage CD-ROM
+/dev/sjcd		Sanyo CD-ROM
+/dev/double0	First compressed disk
+/dev/cdouble0	Mirror of first compressed disk
+/dev/hitcd	Hitachi CD-ROM
+/dev/sg0		First generic SCSI device
+/dev/hdc		Master: whole disk (or CD-ROM)
+/dev/mcd		Mitsumi CD-ROM
+/dev/cdu535	Sony CDU-535 CD-ROM
+/dev/aztcd	Aztech CD-ROM
+/dev/cm205cd	Philips LMS CM-205 CD-ROM
+/dev/eda		First ESDI disk whole disk
+/dev/eza		Parallel EZ135 drive, whole disk
+/dev/cdrom	symbolic Current CD-ROM device
+	***/
+
 		if (strncmp (line, "/dev/sd", 7)
 		    && strncmp (line, "/dev/sr", 7)
 		    && strncmp (line, "/dev/hd", 7)
 		    && strncmp (line, "/dev/xd", 7)
+		    && strncmp (line, "/dev/md", 7)
 		    && strncmp (line, "/dev/fd", 7))
 		  continue;
 		
@@ -777,6 +843,7 @@ var_hr(vp, name, length, exact, var_len, write_method)
 	              ! strncmp (i_line, "/dev/sr", 7) ? 5 :
 		      ! strncmp (i_line, "/dev/hd", 7) ? 4 :   /* XXX: cdrom */
 		      ! strncmp (i_line, "/dev/xd", 7) ? 4 :
+		      ! strncmp (i_line, "/dev/md", 7) ? 4 :
 		      ! strncmp (i_line, "/dev/fd", 7) ? 6 : 1;
 	  }
 
@@ -977,7 +1044,7 @@ var_hr(vp, name, length, exact, var_len, write_method)
 	    */
 
 	    *var_len = sizeof(long);
-	    long_return = get_HrProcessorLoad();
+	    long_return = get_HrProcessorLoad (hr_n_cpu == 1 ? 0 : idx + 1);
 	    return (char *) &long_return;
 #else
 	    /* guess a processorload from the avenrun index (over the
@@ -1027,6 +1094,7 @@ var_hr(vp, name, length, exact, var_len, write_method)
 			&& strncmp (line, "/dev/sr", 7)
 			&& strncmp (line, "/dev/hd", 7)
 			&& strncmp (line, "/dev/xd", 7)
+			&& strncmp (line, "/dev/md", 7)
 			&& strncmp (line, "/dev/fd", 7))
 		      continue;
 	      if (stat (fs_path, &stbuf) < 0)
