@@ -54,7 +54,7 @@ static void sprint_variable();
 
 /* defined types table (from the ASN.1, SMI, RFC 1065) */
 /* only filled as needed */
-static const char *stringtable[] = {
+static char *stringtable[] = {
 "UNIVERSAL PRIMITIVE",	"BOOLEAN",	"INTEGER",	"BIT STRING",
 "OCTET STRING",	"NULL",		"OBJECT IDENTIFIER",	"",
 "",		"",		"",		"",
@@ -91,7 +91,7 @@ NULL
  * "Ipaddress", "Counter", "Gauge", "Timeticks",
  * "Opaque", "NsapAddress", "Counter64", "UInteger32",
  */
-static const char *separator[16] ={
+static char *separator[16] ={
 /* Separator="Wcew" White colon equal white */
 "",	":",	"=",	":=",
 " ",	" :",	" =",	" :=",
@@ -104,7 +104,7 @@ static int AddHexDumpMaxChar = 4;
 static int HideOID = 0;
 static int ShowType = 2;
 
-static const char*timeformat[4] ={
+static char*timeformat[4] ={
   /* Other libraries have the following format strings documented */
   "%ld", "%2$s", "(%ld) %s", NULL
 };
@@ -727,10 +727,14 @@ initmib( cmuoptions )
     Mib = 0;
     file = getenv("MIBFILE");
     if (file) {
-	Mib = read_mib(file);
+      if ((Mib = read_mib(file))) {
+	fprintf (stderr, "info: using MIB-file from environment setting.\n");
+      }
     }
     if (! Mib) {
-	Mib = read_mib("mib.txt");
+      if ((Mib = read_mib("mib.txt"))) {
+	fprintf (stderr, "info: using MIB-file in the working directory.\n");
+      }
     }
 #ifdef MIBFILEPATH
     if (! Mib) {
@@ -740,10 +744,12 @@ initmib( cmuoptions )
     }
 #endif
     if (! Mib) {
-	Mib = read_mib("/etc/mib.txt");
+      if ((Mib = read_mib("/etc/mib.txt"))) {
+	fprintf (stderr, "info: using MIB-file in /etc directory.\n");
+      }
     }
     if (! Mib) {
-	fprintf(stderr, "Couldn't find mib file\n");
+	fprintf(stderr, "Error: couldn't find mib file.\n");
 	exit(2);
     }
 
