@@ -51,7 +51,7 @@ struct subid {
 };
 
 /* use large token buffer in case of very long tokens: */
-#define MAXTC	1024
+#define MAXTC	10240
 struct tc {	/* textual conventions */
     int type;
     char descriptor[MAXTOKEN];
@@ -127,6 +127,7 @@ int Line = 1;
 #define DEFINITIONS 58
 #define END         59
 #define SEMI        60
+#define TRAP_TYPE   61
 
 struct tok {
 	char *name;			/* token name */
@@ -201,6 +202,7 @@ struct tok tokens[] = {
 	{ "(", sizeof ("(")-1, LEFTPAREN },
 	{ ")", sizeof (")")-1, RIGHTPAREN },
 	{ ",", sizeof (",")-1, COMMA },
+	{ "TRAP-TYPE", sizeof ("TRAP-TYPE")-1, TRAP_TYPE },
 	{ NULL }
 };
 
@@ -632,12 +634,14 @@ parse_objectid(fp, name)
     struct subid oid[MAX_OID_LEN];
     struct node *np, *root, *oldnp = NULL;
 
+    bzero(oid, sizeof(oid));
+
     type = get_token(fp, token);
     if (type != EQUALS){
 	print_error("Bad format", token, type);
 	return 0;
     }
-    if ((length = getoid(fp, oid, 32)) != 0){
+    if ((length = getoid(fp, oid, MAX_OID_LEN)) != 0){
 	np = root = (struct node *)Malloc(sizeof(struct node));
 	bzero((char *)np, sizeof(struct node));
 	/*
@@ -1130,8 +1134,8 @@ printf("Description== \"%.50s\"\n", quoted_string_buffer);
 	free_node(np);
 	return 0;
     }
-    length = getoid(fp, oid, 32);
-    if (length > 1 && length <= 32){
+    length = getoid(fp, oid, MAX_OID_LEN);
+    if (length > 1 && length <= MAX_OID_LEN){
 	/* just take the last pair in the oid list */
 	if (oid[length - 2].label)
 	    strncpy(np->parent, oid[length - 2].label, MAXLABEL);
@@ -1585,9 +1589,27 @@ skipget:
 	    goto skipget;
 	} else if (type == ENDOFFILE){
 	    break;
-	} else {
-	    print_error("Bad operator", (char *)NULL, type);
+	} else if (type == TRAP_TYPE){
+	  /* 
+	   * skip over trap-type lines like:
+	   *  label TRAP-TYPE    
+	   *	  ENTERPRISE		foo
+	   *	  VARIABLES		{ bar }
+	   *	  DESCRIPTION		"This trap ..."
+	   *	  ::=			42
+	   */	  
+	  while((type = get_token(fp, token)) != ENDOFFILE && type != EQUALS) 
+	    continue;
+	  if (type != EQUALS) {
+	    print_error ("Expected \"::=\"", NULL, type);
 	    return NULL;
+	  }
+	  if ((type = get_token (fp, token)) != NUMBER){
+	    print_error ("Expected integer", token, type);
+	  }
+	} else {
+	  print_error ("Bad operator", (char *)NULL, type);
+	  return NULL;
 	}
     }
 #ifdef TEST
