@@ -81,9 +81,13 @@ oid default_enterprise[] = {1, 3, 6, 1, 4, 1, 3, 1, 1};
  * Internal information about the state of the snmp session.
  */
 struct snmp_internal_session {
-    int	    sd;		/* socket descriptor for this connection */
-    ipaddr  addr;	/* address of connected peer */
-    struct request_list *requests;/* Info about outstanding requests */
+    int	    sd_ip;		/* socket descriptor for this connection */
+    ipaddr  ip_addr;		/* address of connected peer */
+#ifdef HAVE_IPX
+    int     sd_ipx;		/* socket descriptor for this IPX connection */
+    ipxaddr ipx_addr;		/* address of connected IPX peer */
+#endif
+    struct request_list *requests;	/* Info about outstanding requests */
 };
 
 /*
@@ -161,7 +165,7 @@ static void
 snmp_print_packet(packet, length, addr, code)
   char *packet; 
   int length; 
-  ipaddr addr; 
+  struct sockaddr addr; 
   int code;
 {
   if(length < 0) {
@@ -173,7 +177,16 @@ snmp_print_packet(packet, length, addr, code)
   else { /* sending */
     printf("\nSending  %4d bytes to   ", length);
   }
-  printf("%s:", inet_ntoa(addr.sin_addr));
+#ifdef HAVE_IPX
+  if (addr.sa_family == AF_IPX)
+    printf ("%s:", ipx_ntoa((ipxaddr *)&addr));
+  else 
+#endif
+  if (addr.sa_family == AF_INET)
+    printf ("%s:", inet_ntoa((*(ipaddr *)&addr).sin_addr));
+  else 
+    printf ("(unknown type):");
+
 #if DEBUG_SNMPFULLDUMP
   for (count = 0; count < length; count++) {
     if ((count & 15) == 0) {
@@ -237,6 +250,11 @@ snmp_open(session)
     struct hostent *hp;
     struct servent *servp;
     extern int check_received_pkt();
+    int need_ip, need_ipx;
+#ifdef HAVE_IPX
+    int sd_ipx;
+    struct sockaddr_ipx me_ipx;
+#endif
 
     /* Copy session structure and link into list */
     slp = (struct session_list *)calloc(1, sizeof(struct session_list));
@@ -244,13 +262,18 @@ snmp_open(session)
       calloc(1, sizeof(struct snmp_internal_session));
     slp->internal = isp;
     bzero((char *)isp, sizeof(struct snmp_internal_session));
-    slp->internal->sd = -1; /* mark it not set */
+    slp->internal->sd_ip = -1;		/* mark it not set */
+#ifdef HAVE_IPX
+    slp->internal->sd_ipx = -1;		/* mark it not set */
+#endif
     slp->session = (struct snmp_session *)calloc(1, sizeof(struct snmp_session));
     bcopy((char *)session, (char *)slp->session, sizeof(struct snmp_session));
     session = slp->session;
     /* now link it in. */
     slp->next = Sessions;
     Sessions = slp;
+    need_ip = 0;
+    need_ipx = 0;
 
     /*
      * session now points to the new structure that still contains pointers to
@@ -296,8 +319,89 @@ snmp_open(session)
     /* replace comm pointer with pointer to new data: */
     session->community = cp;	
 
+    /*
+     * setup ports:
+     */
+    if ((session->remote_port != 0) || (session->local_port != 0))
+      need_ip++;
+#ifdef HAVE_IPX
+    if ((session->remote_port_ipx != 0) || (session->local_port_ipx != 0))
+      need_ipx++;
+#endif
+ 
+    if (session->peername != SNMP_DEFAULT_PEERNAME){
+#ifdef HAVE_IPX
+        if (ipx_addr(session->peername, &isp->ipx_addr) != -1)
+        {
+          isp->ipx_addr.sipx_family = AF_IPX;
+          isp->ipx_addr.sipx_type = 4;
+          if(session->remote_port_ipx == SNMP_DEFAULT_REMPORT)
+            isp->ipx_addr.sipx_port = htons(SNMP_PORT_IPX);
+          else
+            isp->ipx_addr.sipx_port = htons(session->remote_port_ipx);
+          need_ipx++;
+        }
+        else
+#endif /* HAVE_IPX */
+        {
+		if ((addr = inet_addr(session->peername)) != -1){
+		    bcopy((char *)&addr, (char *)&isp->ip_addr.sin_addr, 
+			  sizeof(isp->ip_addr.sin_addr));
+		} else {
+		    hp = gethostbyname(session->peername);
+		    if (hp == NULL){
+			fprintf(stderr, "unknown host: %s\n", session->peername);
+			snmp_errno = SNMPERR_BAD_ADDRESS;
+			if (!snmp_close(session)){
+			    fprintf(stderr, "Couldn't abort session: %s. Exiting\n", 
+				    api_errstring(snmp_errno));
+			    exit(2);
+			}
+			return 0;
+		    } else {
+			bcopy((char *)hp->h_addr, (char *)&isp->ip_addr.sin_addr, 
+			      hp->h_length);
+		    }
+		}
+		isp->ip_addr.sin_family = AF_INET;
+		if (session->remote_port == SNMP_DEFAULT_REMPORT){
+		    /*servp = getservbyname("snmp", "udp");*/
+		    servp=NULL;
+		    if (servp != NULL){
+			isp->ip_addr.sin_port = servp->s_port;
+		    } else {
+			isp->ip_addr.sin_port = htons(SNMP_PORT);
+		    }
+		} else {
+		    isp->ip_addr.sin_port = htons(session->remote_port);
+		}
+		need_ip++;
+	}
+     } else {
+	isp->ip_addr.sin_addr.s_addr = SNMP_DEFAULT_ADDRESS;
+#ifdef HAVE_IPX
+	memset(isp->ipx_addr.sipx_node, 0, sizeof(isp->ipx_addr.sipx_node));
+	isp->ipx_addr.sipx_network = 0;
+#endif
+    }
+
+
+    if(need_ipx == 2)
+    {
+      --need_ipx;
+      if(need_ip != 0)
+        --need_ip;
+    } else if(need_ip == 2)
+    {
+      --need_ip;
+      if(need_ipx != 0)
+        --need_ipx;
+    }
+
+    if(need_ip)
+    {
+    /* Set up connections - UDP */
 
-    /* Set up connections */
     sd = socket(AF_INET, SOCK_DGRAM, 0);
     if (sd < 0){
 	perror("socket");
@@ -309,7 +413,7 @@ snmp_open(session)
 	}
 	return 0;
     }
-    isp->sd = sd;
+    isp->sd_ip = sd;
 
 
     /* we'll suggest to reuse the address in the case this is neccessary.
@@ -322,47 +426,56 @@ snmp_open(session)
     }
 #endif
 
-
-    if (session->peername != SNMP_DEFAULT_PEERNAME){
-	if ((addr = inet_addr(session->peername)) != -1){
-	    bcopy((char *)&addr, (char *)&isp->addr.sin_addr, 
-		  sizeof(isp->addr.sin_addr));
-	} else {
-	    hp = gethostbyname(session->peername);
-	    if (hp == NULL){
-		fprintf(stderr, "unknown host: %s\n", session->peername);
-		snmp_errno = SNMPERR_BAD_ADDRESS;
-		if (!snmp_close(session)){
-		    fprintf(stderr, "Couldn't abort session: %s. Exiting\n", 
-			    api_errstring(snmp_errno));
-		    exit(2);
-		}
-		return 0;
-	    } else {
-		bcopy((char *)hp->h_addr, (char *)&isp->addr.sin_addr, 
-		      hp->h_length);
-	    }
-	}
-	isp->addr.sin_family = AF_INET;
-	if (session->remote_port == SNMP_DEFAULT_REMPORT){
-	    /*servp = getservbyname("snmp", "udp");*/
-	    servp=NULL;
-	    if (servp != NULL){
-		isp->addr.sin_port = servp->s_port;
-	    } else {
-		isp->addr.sin_port = htons(SNMP_PORT);
-	    }
-	} else {
-	    isp->addr.sin_port = htons(session->remote_port);
-	}
-    } else {
-	isp->addr.sin_addr.s_addr = SNMP_DEFAULT_ADDRESS;
-    }
-
     me.sin_family = AF_INET;
     me.sin_addr.s_addr = INADDR_ANY;
     me.sin_port = htons(session->local_port);
     if (bind(sd, (struct sockaddr *)&me, sizeof(me)) != 0){
+	perror("bind");
+	snmp_errno = SNMPERR_BAD_LOCPORT;
+	if (! snmp_close(session)){
+		    fprintf(stderr, "Couldn't abort session: %s. Exiting\n", 
+			    api_errstring(snmp_errno));
+		    exit(3);
+		}
+		return 0;
+	    }
+	}
+
+    if(need_ipx)
+    {
+#ifdef HAVE_IPX
+    /* Set up connections - IPX */
+    sd_ipx = socket(PF_IPX, SOCK_DGRAM, PF_IPX);
+    if (sd_ipx < 0){
+	perror("socket");
+	snmp_errno = SNMPERR_GENERR;
+	if (!snmp_close(session)){
+	    fprintf(stderr, "Couldn't abort session: %s. Exiting\n", 
+		    api_errstring(snmp_errno));
+	    exit(1);
+	}
+	return 0;
+	}
+
+    isp->sd_ipx = sd_ipx;
+
+
+    /* we'll suggest to reuse the address in the case this is neccessary.
+     * (and this should not harm) */
+    on = 1;
+#ifdef SO_REUSEADDR
+    if (setsockopt(sd_ipx, SOL_SOCKET, SO_REUSEADDR, (void *) &on, 
+		    sizeof (on)) < 0) {
+      perror ("snmp_open: warning: reuseaddr failed - ignored; reason");
+    }
+#endif
+
+    me_ipx.sipx_family = AF_IPX;
+    memset(me_ipx.sipx_node, 0, sizeof(me_ipx.sipx_node));
+    me_ipx.sipx_network = 0;
+    me_ipx.sipx_type = 4;
+    me_ipx.sipx_port = htons(session->local_port_ipx);
+    if (bind(sd_ipx, (struct sockaddr *)&me_ipx, sizeof(me_ipx)) != 0){
 	perror("bind");
 	snmp_errno = SNMPERR_BAD_LOCPORT;
 	if (! snmp_close(session)){
@@ -372,6 +485,10 @@ snmp_open(session)
 	}
 	return 0;
     }
+#else /* ! HAVE_IPX */
+    fprintf (stderr, "Sorry, IPX is not configured.\n");
+#endif /* HAVE_IPX */
+    }
 
     if( *cp == '/' ) {
 	session->authenticator = check_received_pkt;
@@ -503,8 +620,12 @@ snmp_close(session)
 	if(slp->session->peername != NULL)
 	    free((char *)slp->session->peername);
 	free((char *)slp->session);
-	if (slp->internal->sd != -1)
-	    close(slp->internal->sd);
+	if (slp->internal->sd_ip != -1)
+	    close(slp->internal->sd_ip);
+#ifdef HAVE_IPX
+	if (slp->internal->sd_ipx != -1)
+	    close(slp->internal->sd_ipx);
+#endif
 	free_request_list(slp->internal->requests);
 	free((char *)slp->internal);
 	free((char *)slp);
@@ -806,6 +927,7 @@ snmp_send(session, pdu)
     int length = PACKET_LENGTH;
     struct request_list *rp;
     struct timeval tv;
+    int rc;
 
     for(slp = Sessions; slp; slp = slp->next){
 	if (slp->session == session){
@@ -843,30 +965,47 @@ snmp_send(session, pdu)
 	if (pdu->time == SNMP_DEFAULT_TIME)
 	    pdu->time = DEFAULT_TIME;
     }
-    if (pdu->address.sin_addr.s_addr == SNMP_DEFAULT_ADDRESS){
-	if (isp->addr.sin_addr.s_addr != SNMP_DEFAULT_ADDRESS){
-	    bcopy((char *)&isp->addr, (char *)&pdu->address, sizeof(pdu->address));
+#ifdef HAVE_IPX
+    if (pdu->address.sa_family == AF_UNSPEC) {
+#endif
+	if (isp->ip_addr.sin_addr.s_addr != SNMP_DEFAULT_ADDRESS){
+	    bcopy((char *)&isp->ip_addr, (char *)&pdu->address, 
+		  sizeof(pdu->address));
+#ifdef HAVE_IPX
+	} else if(isp->ipx_addr.sipx_family == AF_IPX){
+	    bcopy((char *)&isp->ipx_addr, (char *)&pdu->address, 
+		  sizeof(pdu->address));
+#endif
 	} else {
 	    fprintf(stderr, "No remote IP address specified\n");
 	    snmp_errno = SNMPERR_BAD_ADDRESS;
 	    return 0;
 	}
+#ifdef HAVE_IPX
     }
-	
+#endif
 
     if (snmp_build(session, pdu, packet, &length, 0) < 0){
 	fprintf(stderr, "Error building packet\n");
 	snmp_errno = SNMPERR_GENERR;
 	return 0;
     }
-#if 1 /***/
+
     if (snmp_dump_packet){
       snmp_print_packet(packet, length, pdu->address, 1);
     }
-#endif /***/
 
     gettimeofday(&tv, (struct timezone *)0);
-    if (sendto(isp->sd, (char *)packet, length, 0, (struct sockaddr *)&pdu->address, sizeof(pdu->address)) < 0){
+#ifdef HAVE_IPX
+    rc = sendto((pdu->address.sa_family == AF_INET 
+		 ? isp->sd_ip : isp->sd_ipx), 
+		(char *)packet, length, 0, 
+		(struct sockaddr *)&pdu->address, sizeof(pdu->address));
+#else
+    rc = sendto(isp->sd_ip, (char *)packet, length, 0, 
+		(struct sockaddr *)&pdu->address, sizeof(pdu->address));
+#endif
+  if (rc < 0) {
 	perror("sendto");
 	snmp_errno = SNMPERR_GENERR;
 	return 0;
@@ -947,17 +1086,29 @@ snmp_read(fdset)
     struct snmp_session *sp;
     struct snmp_internal_session *isp;
     u_char packet[PACKET_LENGTH];
-    struct sockaddr_in	from;
+    struct sockaddr from;
     ssize_t length, fromlength;
     struct snmp_pdu *pdu;
     struct request_list *rp /**, *orp **/ ;
+    int sd;
 
     for(slp = Sessions; slp; slp = slp->next){
-	if (FD_ISSET(slp->internal->sd, fdset)){
+	if ((slp->internal->sd_ip != -1) && FD_ISSET(slp->internal->sd_ip, fdset))
+	  sd = slp->internal->sd_ip;
+#ifdef HAVE_IPX
+	else if((slp->internal->sd_ipx != -1) && FD_ISSET(slp->internal->sd_ipx, fdset))
+	  sd = slp->internal->sd_ipx;
+#endif
+	else
+	  sd = -1;
+
+	if(sd != -1)
+	{
 	    sp = slp->session;
 	    isp = slp->internal;
 	    fromlength = sizeof(from);
-	    length = recvfrom(isp->sd, (char *)packet, PACKET_LENGTH, 0, (struct sockaddr *)&from, &fromlength);
+	    length = recvfrom(sd, (char *)packet, PACKET_LENGTH, 0, 
+			      (struct sockaddr *)&from, &fromlength);
 	    if (length == -1) {
 		perror("recvfrom");
 		return;
@@ -975,7 +1126,11 @@ snmp_read(fdset)
 	    }
 #endif /***/
 
+#ifdef HAVE_IPX
 	    pdu->address = from;
+#else
+	    pdu->address = *(struct sockaddr_in *) &from;
+#endif
 	    pdu->reqid = 0;
 	    pdu->variables = NULL;
 	    pdu->enterprise = NULL;
@@ -1062,9 +1217,21 @@ snmp_select_info(numfds, fdset, timeout, block)
     for(slp = Sessions; slp; slp = slp->next){
 	active++;
 	isp = slp->internal;
-	if ((isp->sd + 1) > *numfds)
-	    *numfds = (isp->sd + 1);
-	FD_SET(isp->sd, fdset);
+
+	if(isp->sd_ip != -1)
+	{
+	    if ((isp->sd_ip + 1) > *numfds)
+	        *numfds = (isp->sd_ip + 1);
+	    FD_SET(isp->sd_ip, fdset);
+	}
+#ifdef HAVE_IPX
+	if(isp->sd_ipx != -1)
+	{
+	    if ((isp->sd_ipx + 1) > *numfds)
+	        *numfds = (isp->sd_ipx + 1);
+	    FD_SET(isp->sd_ipx, fdset);
+	}
+#endif
 	if (isp->requests){
 	    /* found another session with outstanding requests */
 	    requests++;
@@ -1124,6 +1291,7 @@ snmp_timeout()
     struct request_list *rp, *rp_next = 0;
 #endif /***/
     struct timeval now;
+    int rc;
 
     gettimeofday(&now, (struct timezone *)0);
     /*
@@ -1164,7 +1332,18 @@ snmp_timeout()
 		    }
 #endif /***/
 		    gettimeofday(&tv, (struct timezone *)0);
-		    if (sendto(isp->sd, (char *)packet, length, 0, (struct sockaddr *)&rp->pdu->address, sizeof(rp->pdu->address)) < 0){
+#ifdef HAVE_IPX
+		    rc = sendto((rp->pdu->address.sa_family == AF_INET 
+				 ? isp->sd_ip : isp->sd_ipx), 
+				(char *)packet, length, 0, 
+				(struct sockaddr *)&rp->pdu->address, 
+				sizeof(rp->pdu->address));
+#else
+		    rc = sendto(isp->sd_ip, (char *)packet, length, 0, 
+				(struct sockaddr *)&rp->pdu->address, 
+				sizeof(rp->pdu->address));
+#endif
+		    if (rc < 0) {
 			perror("sendto");
 		    }
 		    rp->time = tv;
@@ -1184,3 +1363,57 @@ snmp_timeout()
 }
 
 
+#ifdef HAVE_IPX
+char *ipx_ntoa(struct sockaddr_ipx *sipx)
+{
+  static char s[24];
+  
+  sprintf(s, "%08X:%02X%02X%02X%02X%02X%02X", ntohl(sipx->sipx_network),
+         sipx->sipx_node[0],
+         sipx->sipx_node[1],
+         sipx->sipx_node[2],
+         sipx->sipx_node[3],
+         sipx->sipx_node[4],
+         sipx->sipx_node[5]);
+
+  return(s);
+}
+
+int ipx_addr(const char *str, ipxaddr *sipx)
+{
+  char *s, str1[40];
+  long n;
+  int i, j;
+  unsigned char x;
+  
+  strncpy(str1, str, sizeof(str1));
+  str1[sizeof(str1) - 1] = '\0';
+
+  if((s = strchr(str1, ':')) == (char *)NULL)
+    return(-1);
+  if(sscanf(str1, "%lX:", &n) != 1)
+    return(-1);
+  sipx->sipx_network = htonl(n);
+  memset(sipx->sipx_node, 0, sizeof(sipx->sipx_node));
+  for(i = 0; i < IPX_NODE_LEN * 2; ++i)
+  {
+    ++s;
+    if(!isxdigit(*(unsigned char *)s))
+      break;
+    x = toupper(*(unsigned char *)s);
+    if(isdigit(x))
+      x -= '0';
+    else
+      x -= 'A' - 10;
+    for(j = 0; j < IPX_NODE_LEN - 1; ++j)
+      sipx->sipx_node[j] = (sipx->sipx_node[j] << 4) || (sipx->sipx_node[j + 1] >> 4);
+    sipx->sipx_node[j] = (sipx->sipx_node[j] << 4) | x;
+  }
+  if((i == 0) || ((*s != ' ') && (*s != '\0')))
+    return(-1);
+  sipx->sipx_family = AF_IPX;
+  return(0);
+}
+#endif /* HAVE_IPX */
+
+/* end of snmp_api.c */
